#!/bin/bash
#------------------------------------------------------------------
#                        C O M O D O R O  2.0
#
# Conversión de ficheros .T64, .PRG y .P00 a .TAP y .TZX para orde- 
# nadores de Commodore 8 bits: PET, VIC20, C64 y C16/116/Plus4.
#
# Instrucciones: Colocar los ficheros .T64, .PRG, .P00 y .TAP 
# dentro de las carpetas <máquina>/{T64 | PRG | P00 | TAP}, (se 
# pueden añadir subcarpetas) dónde <máquina> puede ser PET, VIC, 
# C64 o C16.
#
# Al ejecutar este script se debe indicar si la conversión se
# quiere realizar a ficheros .TAP de Commodore, a ficheros .TZX
# de Sinclair Spectrum, o a ambos. Los .TAP se pueden cargar con
# un TAPuino o con un TZXduino / Miniduino / Maxduino  al que se
# le haya instalado el firmware TapuinoOnTZXduino. Los .TZX 
# se pueden reproducir en un TZXduino / Miniduino / Maxduino con
# el firmware Maxduino de Rafaél Molina.
#
# Los archivos .TAP convertidos se grabarán en la carpeta TAP-CONV,
# y los .TZX en la carpeta TZX, clonándose también las estructura
# de subcarpetas de las carpetas origen.
#
# Si los ficheros destino ya existiesen no se volverán a convertir,
# excepto si se llama al script con el parámetro -f
#-----------------------------------------------------------------



function ayuda(){

    echo "Conversión de ficheros .T64, .PRG y .P00 a .TAP y .TZX para ordenadores de Commodore 8 bits: PET, VIC20, C64 y C16/116/Plus4."
    echo
    echo "Modo de empleo: comodoro.sh [opciones] MODELO [conversiones]"
    echo
    echo "Los valores admitidos para MODELO son: c64pal, c64ntsc, c16pal, c16ntsc, vicpal, vicntsc y pet."
    echo
    echo "Opciones (solo se puede usar una):"
    echo
    echo "-h , --help: "
    echo "     Presenta esta ayuda."
    echo "-v , --version: "
    echo "     Imprime la versión del script."
    echo "-f , --forzar: "
    echo "     Si se añade se sobrescribirán los ficheros destinos aunque ya existan."
    echo
    echo "Si la conversión de se procesa sin errores se puede ver el resultado en el fichero comodoro.log"     
    echo
    echo
    echo "Conversiones (solo se puede usar una como máximo, si no se informa se asume -tap):"
    echo
    echo "-tap , --conv2tap: "
    echo "     Realiza la conversión al formato .TAP de commodore."
    echo "-tzx , --conv2tzx: "
    echo "     Realiza la conversión al formato .TZX de Sinclair Spectrum."
    echo "-dual , --conv-dual: "
    echo "     Realiza la conversión a los formatos .TAP y .TZX."
    echo
    echo "Intrucciones:"
    echo
    echo "Colocar los ficheros .T64, .PRG, .P00 y .TAP dentro de las carpetas <máquina>/{T64 | PRG | P00 | TAP}, (se pueden"
    echo "añadir subcarpetas) dónde <máquina> puede ser PET, VIC, C64 o C16. Al ejecutar este script se puede indicar si la " 
    echo "conversión se quiere realizar a ficheros .TAP de Commodore, a ficheros .TZX de Sinclair Spectrum, o a ambos."
    echo
    echo "Los .TAP se pueden cargar con un TAPuino o con un TZXduino / Miniduino / Maxduino  al que se le haya instalado el"
    echo "firmware TapuinoOnTZXduino. Los .TZX se pueden reproducir en un TZXduino / Miniduino / Maxduino con el firmware"
    echo "Maxduino de Rafaél Molina."
    echo 
    echo "Los archivos .TAP convertidos se grabarán en la carpeta TAP-CONV, y los .TZX en la carpeta TZX, clonándose también"
    echo "la estructura de subcarpetas de las carpetas origen."
    echo 
    echo "Si los ficheros destino ya existiesen no se volverán convertir, excepto si se llama al script con el parámetro -f"    
    echo
    echo "Ejemplos:"
    echo
    echo "./comodoro.sh c64pal"    
    echo
    echo "     Convertirá los archivos .T64, .PRG y .P00 que encuentre dentro de las carpetas ./C64/T64, ./C64/PRG y ./C64/P00"
    echo "     a ficheros .TAP de Commodore 64 modelo PAL dentro de la carpeta ./C64/TAP, manteniendo la estructura de carpetas."
    echo "     Si se encuentra que el fichero destino ya existe no realiza la conversión del fichero origen del mismo nombre."
    echo
    echo "./comodoro.sh -f c64pal -tzx"    
    echo
    echo "     Igual que en el ejemplo anterior, pero en este caso convertirá los ficheros origen a formato .TZX aunque haya"
    echo "     otros en destino con el mismo nombre, sobrescribiéndolos."
    exit
}

ver="2.0"
dir_progs="bin/"
opciones=0
modelos=0
conversiones=0
convertidos=0
no_convertidos=0

# Control de los argumentos de entrada

if [ $# -eq 0 ]; then
    ayuda
else
    for arg in "$@" #en todos los argumentos que han entrado
    do    
        if [ "$arg" != "--help" ]    && [ "$arg" != "-h" ]      &&
           [ "$arg" != "--forzar" ]  && [ "$arg" != "-f" ]      &&
           [ "$arg" != "--version" ] && [ "$arg" != "-v" ]      &&           
           [ "$arg" != "--conv2tap" ]  && [ "$arg" != "-tap" ]  &&
           [ "$arg" != "--conv2tzx" ]  && [ "$arg" != "-tzx" ]  &&
           [ "$arg" != "--conv-dual" ] && [ "$arg" != "-dual" ] &&           
           [ "$arg" != "c64pal" ]    && [ "$arg" != "c64ntsc" ] &&
           [ "$arg" != "c16pal" ]    && [ "$arg" != "c16ntsc" ] &&
           [ "$arg" != "vicpal" ]    && [ "$arg" != "vicntsc" ] &&
           [ "$arg" != "pet" ]; then
           echo "Parámetro erróneo:  $arg" 
           exit
        else   
            if [ "$arg" == "--help" ] || [ "$arg" == "-h" ]; then
                ayuda=true            
                ((opciones += 1))
            elif [ "$arg" == "--forzar" ] || [ "$arg" == "-f" ]; then
                forzar=true      
                ((opciones += 1))
            elif [ "$arg" == "--version" ] || [ "$arg" == "-v" ]; then
                version=true       
                ((opciones += 1))
            elif [ "$arg" == "--conv2tap" ] || [ "$arg" == "-tap" ]; then
                conversion=1       
                ((conversiones += 1))    
            elif [ "$arg" == "--conv2tzx" ] || [ "$arg" == "-tzx" ]; then
                conversion=2       
                ((conversiones += 1))        
            elif [ "$arg" == "--conv-dual" ] || [ "$arg" == "-dual" ]; then
                conversion=3       
                ((conversiones += 1))                
            elif [ "$arg" == "c64pal" ] || [ "$arg" == "c64ntsc" ]; then
                modelo="$arg"
                maquina="C64"
                ((modelos += 1))
            elif [ "$arg" == "c16pal" ] || [ "$arg" == "c16ntsc" ]; then
                modelo="$arg"
                maquina="C16"
                ((modelos += 1)) 
            elif [ "$arg" == "vicpal" ] || [ "$arg" == "vicntsc" ]; then
                modelo="$arg"
                maquina="VIC"
                ((modelos += 1))                          
            elif [ "$arg" == "pet" ]; then
                modelo="$arg"
                maquina="PET"
                ((modelos += 1))                          
           fi     
        fi                
    done           
        
    if [ "$opciones" -gt 1 ]; then
        echo "No ponga más de una opción a la vez"
        exit
    elif [ "$ayuda" == true ]; then
        if [ $# -eq 1 ]; then
           ayuda
        else
          echo "Si quiere ver la ayuda ponga solo la opción -h o --help"
          exit        
        fi
    elif [ "$version" == true ]; then
        if [ $# -eq 1 ]; then
           echo "Versión $ver"
           exit
        else
          echo "Si quiere ver la versión ponga solo la opción -v o --version"
          exit        
        fi
    fi
                       
    if [ "$modelos" -eq "0" ]; then
        echo "No se ha informado un modelo correcto. Los posibles modelos son: c64pal, c64ntsc, c16pal, c16ntsc, vicpal, vicntsc y pet."
        exit
    elif [ "$modelos" -ne "1" ]; then
        echo "Ha introducido más de un modelo."
        exit    
    fi    
    
    if [ "$conversiones" -eq "0" ]; then
        conversion=1        
    elif [ "$conversiones" -gt "1" ]; then
        echo "Ha introducido más de un tipo de conversion."
        exit    
    fi        
fi

# Validaciones adicionales

if ([ "$maquina" == "C64" ] && [ ! -d "C64" ]) || 
   ([ "$maquina" == "C16" ] && [ ! -d "C16" ]) ||  
   ([ "$maquina" == "VIC" ] && [ ! -d "VIC" ]) ||
   ([ "$maquina" == "PET" ] && [ ! -d "PET" ]); then
        echo "No existe el directorio $maquina"
        exit
fi

if ([ ! -d "$maquina/T64" ]) &&
   ([ ! -d "$maquina/PRG" ]) &&
   ([ ! -d "$maquina/TAP" ]) &&
   ([ ! -d "$maquina/P00" ]); then
        echo "No existe ningún subdirectorio T64, PRG, P00 o TAP dentro de carpeta $maquina"
        exit
fi


# Empezamos con la conversión 

# Ficheros temporales

log_convertidos="/tmp/comodoro_log_convertidos.txt"
log_no_convertidos="/tmp/comodoro_log_no_convertidos.txt"
ficheros="/tmp/comodoro_ficheros.txt"

log_proceso="${0//.sh/.log}"
rm -f $log_proceso
touch $log_proceso $log_convertidos $log_no_convertidos $ficheros

echo "*********************************************" >> $log_proceso
echo " " >> $log_proceso
echo "             C O M O D O R O   2.0" >> $log_proceso
echo " " >> $log_proceso
echo "  Conversión de ficheros para modelo $modelo " >> $log_proceso
echo " " >> $log_proceso
echo "*********************************************" >> $log_proceso
echo " " >> $log_proceso

if [ "$conversion" -eq 1 ] || [ "$conversion" -eq 3 ]; then 

   # Buscamos ficheros .T64, .PRG y P00 dentro de la estructura de carpetas que cuelga de <maquina>/{T64 | PRG | P00}     
   
   rm -f $ficheros

   if [ -d "./$maquina/T64" ] ; then
       find ./$maquina/T64 -name "*.t64" -or -name "*.T64" -not -name ".*" -not -name "." | egrep -v "\.rtfd/|\.app/|\.lpdf/|\.workflow/" >>  $ficheros
   fi

   if [ -d "./$maquina/PRG" ] ; then
       find ./$maquina/PRG -name "*.prg" -or -name "*.PRG" -not -name ".*" -not -name "." | egrep -v "\.rtfd/|\.app/|\.lpdf/|\.workflow/" >>  $ficheros
   fi 

   if [ -d "./$maquina/P00" ] ; then
       find ./$maquina/P00 -name "*.p00" -or -name "*.P00" -not -name ".*" -not -name "." | egrep -v "\.rtfd/|\.app/|\.lpdf/|\.workflow/" >>  $ficheros
   fi    

# Se convierten todos los .T64, .PRG y P00 encontrados a .TAP

   while IFS= read -r fichero_origen
   do

     # Nos quedamos con el nombre del fichero .T64, .PRG o .P00 sin la extensión
  
     nombre=`basename "${fichero_origen%.*}"`
  
     # Creamos la ruta y fichero destino cambiando la carpeta /T64/,
     # /PRG/ o /P00/ por /TAP-CONV/, y la extensión de los ficheros a .TAP
  
     fich_temp1="${fichero_origen//\/T64\//\/TAP-CONV\/}"
     fich_temp2="${fich_temp1//\/PRG\//\/TAP-CONV\/}"
     fich_temp1="${fich_temp2//\/P00\//\/TAP-CONV\/}"
     fich_temp2="${fich_temp1//.T64/(T64).tap}"
     fich_temp1="${fich_temp2//.t64/(T64).tap}"
     fich_temp2="${fich_temp1//.PRG/(PRG).tap}"
     fich_temp1="${fich_temp2//.prg/(PRG).tap}" 
     fich_temp2="${fich_temp1//.P00/(P00).tap}"
     fichero_destino="${fich_temp2//.p00/(P00).tap}"
    
     # Si el fichero ya ha sido convertido antes no se vuelve a convertir  
     # a menos que se haya pasado el parámetro -f (forzar)
     
     if [ -f "$fichero_destino" -a  "$forzar" != true ]; then  
  
       # Aumentamos el contador de ficheros no convertidos 
    
       ((no_convertidos += 1))
    
       # Reflejamos en el log que no realizamos la conversión de ese fichero
    
       echo "Ya existe $fichero_destino. No se crea." >> $log_no_convertidos
    
     else
  
       # Comprobamos si ya existe la carpeta destino, y en caso de 
       # que no exista la creamos, incluidas carpetas de niveles 
       # superiores que tampoco existan 
    
       directorio_destino="${fichero_destino%/*}"   
       if [ ! -d "$directorio_destino" ]; then        
         mkdir -p "$directorio_destino"
       fi    
    
       # Realizamos la conversión del fichero origen .T64, rm -f $ficheros.PRG o P00 al formato destino .TAP
    
       if [ "$modelo" == "c64pal" ]; then
    
           # Como c64pal es el modelo por defecto no hay que informarlo con el parámetro -m
        
           wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -T 140 -t "$fichero_destino" "$fichero_origen"            
        
       elif [ "$modelo" == "pet" ]; then
    
           # El PET utiliza las mismas rutinas de cinta que el VIC, pero no se puede usar turbo, así que 
           # hay que utilizar el parámetro -s (slow). 
    
           wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -m vicntsc -s -t "$fichero_destino" "$fichero_origen"
        
        
       elif [ "$maquina" == "VIC" ]; then
    
           # Con el VIC reduzco la velocidad turbo a -T 160
            
           wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -m "$modelo" -T 160 -t "$fichero_destino" "$fichero_origen"
        
        
         else  
      
           # El resto de modelos van en modo turbo con -T 140
      
           wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -m "$modelo" -T 140 -t "$fichero_destino" "$fichero_origen"
                
         fi    
      
    
       # Aumentamos el contador de ficheros convertidos 
    
       ((convertidos += 1))
    
       # Reflejamos en el log el fichero convertido    
    
       echo "Creado $fichero_destino"  >> $log_convertidos
     fi
  
  done < $ficheros
fi


if [ "$conversion" -eq 2 ] || [ "$conversion" -eq 3 ]; then

   # Buscamos ficheros .T64, .PRG, .P00 y .TAP dentro de la estructura de carpetas que cuelga de <maquina>/{T64 | PRG | P00 | TAP-ORIG}
   
   rm -f $ficheros

   if [ -d "./$maquina/T64" ] ; then
       find ./$maquina/T64 -name "*.t64" -or -name "*.T64" -not -name ".*" -not -name "." | egrep -v "\.rtfd/|\.app/|\.lpdf/|\.workflow/" >>  $ficheros
   fi

   if [ -d "./$maquina/PRG" ] ; then
       find ./$maquina/PRG -name "*.prg" -or -name "*.PRG" -not -name ".*" -not -name "." | egrep -v "\.rtfd/|\.app/|\.lpdf/|\.workflow/" >>  $ficheros
   fi 

   if [ -d "./$maquina/P00" ] ; then
       find ./$maquina/P00 -name "*.p00" -or -name "*.P00" -not -name ".*" -not -name "." | egrep -v "\.rtfd/|\.app/|\.lpdf/|\.workflow/" >>  $ficheros
   fi    
   
   if [ -d "./$maquina/TAP" ] ; then
       find ./$maquina/TAP -name "*.tap" -or -name "*.TAP" -not -name ".*" -not -name "." | egrep -v "\.rtfd/|\.app/|\.lpdf/|\.workflow/" >>  $ficheros
   fi

# Se convierten todos los .T64, .PRG, .P00 y .TAP encontrados a .TZX

   while IFS= read -r fichero_origen
   do

     # Nos quedamos con el nombre del fichero .T64, .PRG , .P00 o .TAP sin la extensión
  
     nombre=`basename "${fichero_origen%.*}"`
  
     # Creamos la ruta y fichero destino cambiando la carpeta /T64/,
     # /PRG/, /P00/ o /TAP/ por /TZX/, y la extensión de los ficheros a .TZX
  
     fich_temp1="${fichero_origen//\/T64\//\/TZX\/}"
     fich_temp2="${fich_temp1//\/PRG\//\/TZX\/}"
     fich_temp1="${fich_temp2//\/P00\//\/TZX\/}"
     fich_temp2="${fich_temp1//\/TAP\//\/TZX\/}"     
     fich_temp1="${fich_temp2//.T64/(T64).tzx}"
     fich_temp2="${fich_temp1//.t64/(T64).tzx}"
     fich_temp1="${fich_temp2//.PRG/(PRG).tzx}"
     fich_temp2="${fich_temp1//.prg/(PRG).tzx}" 
     fich_temp1="${fich_temp2//.P00/(P00).tzx}"
     fich_temp2="${fich_temp1//.p00/(P00).tap}"          
     fich_temp1="${fich_temp2//.tap/(TAP).tzx}"
     fichero_destino="${fich_temp1//.TAP/(TAP).tzx}"     
    
     # Si el fichero ya ha sido convertido antes no se vuelve a convertir  
     # a menos que se haya pasado el parámetro -f (forzar)
     
     if [ -f "$fichero_destino" -a  "$forzar" != true ]; then  
  
       # Aumentamos el contador de ficheros no convertidos 
    
       ((no_convertidos += 1))
    
       # Reflejamos en el log que no realizamos la conversión de ese fichero
    
       echo "Ya existe $fichero_destino. No se crea." >> $log_no_convertidos
    
     else
  
       # Comprobamos si ya existe la carpeta destino, y en caso de 
       # que no exista la creamos, incluidas carpetas de niveles 
       # superiores que tampoco existan 
    
       directorio_destino="${fichero_destino%/*}"   
       if [ ! -d "$directorio_destino" ]; then        
         mkdir -p "$directorio_destino"
       fi    
                     
       extension_origen="${fichero_origen##*.}"
       
    
       # Realizamos la conversión del fichero origen .T64, .PRG,  P00 o .TAP al formato destino .TZX
    
       if [ "$modelo" == "c64pal" ]; then
       
           if [ "$extension_origen" == "tap" ] || [ "$extension_origen" == "TAP" ]; then
           
              wine ${dir_progs}tap2audio.exe -f 11025 -i "$fichero_origen" ${dir_progs}comodoro.wav
              wine ${dir_progs}soX.exe ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
              wine ${dir_progs}DIRECT.exe ${dir_progs}comodoro.voc "$fichero_destino"
           
           else
           
              # Como c64pal es el modelo por defecto no hay que informarlo con el parámetro -m
        
              #wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -T 250 -i -w ${dir_progs}comodoro.wav "$fichero_origen"
              
              wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -T 250 -t ${dir_progs}comodoro.tap "$fichero_origen"
              wine ${dir_progs}tap2audio.exe -f 11025 -i ${dir_progs}comodoro.tap ${dir_progs}comodoro.wav              
              wine ${dir_progs}soX.exe ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
              wine ${dir_progs}DIRECT.exe ${dir_progs}comodoro.voc "$fichero_destino"
              
           fi
           
       elif [ "$modelo" == "pet" ]; then
       
           if [ "$extension_origen" == "tap" ] || [ "$extension_origen" == "TAP" ]; then
           
                wine ${dir_progs}tap2audio.exe "$fichero_origen" ${dir_progs}comodoro.wav
                wine ${dir_progs}soX.exe ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
                wine ${dir_progs}DIRECT.exe /t 316 ${dir_progs}comodoro.voc "$fichero_destino"
           
           else
    
                # El PET utiliza las mismas rutinas de cinta que el VIC, pero no se puede usar turbo, así que 
                # hay que utilizar el parámetro -s (slow), y no invertimos la onda a diferencia del resto 
                # (necesario para hacer las cargas en el ZX-Uno)
                
                    
                wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -m vicntsc -s -t ${dir_progs}comodoro.tap "$fichero_origen"                
                wine ${dir_progs}tap2audio.exe -f 11025 ${dir_progs}comodoro.tap ${dir_progs}comodoro.wav
                wine ${dir_progs}soX.exe ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
                wine ${dir_progs}DIRECT.exe ${dir_progs}comodoro.voc "$fichero_destino"
                            
           fi
           
        
       elif [ "$maquina" == "VIC" ]; then
    
           
           if [ "$extension_origen" == "tap" ] || [ "$extension_origen" == "TAP" ]; then
           
                wine ${dir_progs}tap2audio.exe -i "$fichero_origen" ${dir_progs}comodoro.wav
                wine ${dir_progs}soX.exe ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
                wine ${dir_progs}DIRECT.exe /t 316 ${dir_progs}comodoro.voc "$fichero_destino"                      
                
           else 
           
                wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -m "$modelo" -i -T 250 -w ${dir_progs}comodoro.wav "$fichero_origen"
                wine ${dir_progs}soX.exe ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
                wine ${dir_progs}DIRECT.exe /t 316 ${dir_progs}comodoro.voc "$fichero_destino"                            
                
           fi        
         else 
         
           if [ "$extension_origen" == "tap" ] || [ "$extension_origen" == "TAP" ]; then
           
                wine ${dir_progs}tap2audio.exe -f 11025 -i "$fichero_origen" ${dir_progs}comodoro.wav
                wine ${dir_progs}soX.exe ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
                wine ${dir_progs}DIRECT.exe ${dir_progs}comodoro.voc "$fichero_destino"
           
           else
      
                # El resto de modelos van en modo turbo con -T 250
              
                wine ${dir_progs}prg2wav.exe --use-file-name "$nombre" -m "$modelo" -T 250 -t ${dir_progs}comodoro.tap "$fichero_origen"
                wine ${dir_progs}tap2audio.exe -f 11025 -i ${dir_progs}comodoro.tap ${dir_progs}comodoro.wav   
                wine ${dir_progs}soX.exe ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
                wine ${dir_progs}DIRECT.exe ${dir_progs}comodoro.voc "$fichero_destino"
              
           fi                
         fi             
           
       # Aumentamos el contador de ficheros convertidos 
    
       ((convertidos += 1))
    
       # Reflejamos en el log el fichero convertido    
    
       echo "Creado $fichero_destino"  >> $log_convertidos
     fi
  
  done < $ficheros
fi

#Confeccionamos el log final con el resultado

if [ "$convertidos" -ne "0" ]; then  
    cat $log_convertidos >> $log_proceso
    echo " "   >> $log_proceso
fi

if [ "$no_convertidos" -ne "0" ]; then  
    cat $log_no_convertidos >> $log_proceso
    echo " "   >> $log_proceso
fi    


# Reflejamos en el log el número de ficheros que se han convertido

echo "Número de archivos convertidos: $convertidos"         >> $log_proceso
echo "Número de archivos no convertidos: $no_convertidos"   >> $log_proceso

echo "Conversión finalizada. Puede ver el resultado del proceso en $log_proceso"

# Borramos ficheros temporales

rm -f $ficheros $log_convertidos $log_no_convertidos ${dir_progs}comodoro.tap ${dir_progs}comodoro.wav ${dir_progs}comodoro.voc
